#!/usr/bin/env Rscript
#
# MotFall - domain falling along sequences,
# search for domain positions in a given sequence.
# (C) Martin Saturka, http://www.bioplexity.org/
# This script is available under the MIT license.
#
# Script parameters:
# domain.fa   .. domain description, fasta format
# sequence.fa .. scanned sequence, fasta format
# fits.csv    .. point-to-point fit values, csv format
# results.csv .. result file
#

args <- commandArgs(TRUE)
if (4 != length(args)) {
  stop("should be run: motfall.R domain.fa sequence.fa fits.csv results.csv")
}

dseq <- read.table(args[1], comment.char=">")
pseq <- read.table(args[2], comment.char=">")
tfit.df <- read.csv(args[3], row.names=1)

tfit <- as.matrix(tfit.df)
if (!is.integer(tfit)) {
  if (!is.double(tfit)) {
    stop("non-numeric fit values")
  }
}

rldom <- unlist(strsplit(as.vector(dseq$V1), ""))
rlpro <- unlist(strsplit(as.vector(pseq$V1), ""))

dnames <- dimnames(tfit)[[1]]
pnames <- dimnames(tfit)[[2]]

ildom <- match(rldom, dnames, nomatch=0)
ilpro <- match(rlpro, pnames, nomatch=0)

if (0 == min(ildom)) {
  stop("unknown domain character")
}
if (0 == min(ilpro)) {
  stop("unknown sequence character")
}

dalts <- dim(tfit)[1]
palts <- dim(tfit)[2]

domlen <- length(ildom)
prolen <- length(ilpro)

domfit <- t(tfit[ildom,])

fit.func <- function(x) {
  ind <- array(c(ilpro[x:(x+domlen-1)], c(1:domlen)), dim = c(domlen, 2))
  sum(domfit[ind])
}

dfits <- apply(as.matrix(c(1:(prolen+1-domlen))), 1, fit.func)
dfitslen <- length(dfits)

fsorted <- sort(dfits, decreasing=TRUE, index.return=TRUE)

res <- cbind(as.matrix(fsorted$ix), as.matrix(fsorted$x))
dimnames(res) <- list(c(1:dfitslen), c("index", "fit"))

options("warn" = -1)
write.table(res, args[4], append=TRUE, sep=",", col.names=TRUE, row.names=FALSE)


