#!/usr/bin/env python
#
# (C) Martin Saturka, http://www.bioplexity.org/
# This script is available under the MIT license.
#
# Script parameters:
# file.pdb .. pdb-formated structure file
# A .. C-Alpha atoms only, with 'CA' id ones
# B .. Protein Backbone atoms, the first 4 ones
# C .. including Side Chains, all non-H ones
# D .. all the atom/hetam Data lines
# chain list .. colon (:) separated list
#   - no list for usage of all the chains
#   - ':' spec. for empty name chains only
# start residuum .. first used residuum rank
# last residuum  .. last used residuum rank
#   - 0 for no residuum limit
#

import sys, os, time
import getopt

class SelCoords(object):
  def __init__(self):
    self.pdbfname = "input.pdb"
    self.atomtype = "A"
    self.chainlist = []
    self.residstart = 0
    self.residlast = 0

  def work(self):
    try:
      fhpdb = open(self.pdbfname, "r")
    except:
      return False

    chain_limit = False
    if 0 < len(self.chainlist):
      chain_limit = True

    cur_resid_atom = 0
    cur_resid_rank = 0
    cur_chain_symb = "none"

    take_all = False
    if "D" == self.atomtype:
      take_all = True

    cawas = False
    for line in fhpdb:
      indata = line.rstrip()

      if indata.startswith("HEADER"):
        print("#" + indata)
        continue

      curlinetype = ''
      if indata.startswith("ATOM"):
        curlinetype = 'A'
      if take_all:
        if indata.startswith("HETATM"):
          curlinetype = 'H'
      if '' == curlinetype:
        continue

      atom_data = True

      new_atom_rank = 0
      value = (indata[6:6+5]).strip()
      if value.isdigit():
        new_atom_rank = int(value)
      else:
        atom_data = False

      new_atom_symb = ""
      value = (indata[13:13+3]).strip()
      if 0 < len(value):
        new_atom_symb = value
      else:
        atom_data = False

      new_resid_rank = 0
      value = (indata[22:22+4]).strip()
      if value.isdigit():
        new_resid_rank = int(value)
      else:
        atom_data = False
      if 0 == new_resid_rank:
        atom_data = False

      new_resid_symb = ""
      value = (indata[17:17+3]).strip()
      if 0 < len(value):
        new_resid_symb = value
      else:
        atom_data = False

      new_chain_symb = ""
      value = (indata[20:20+2]).strip()
      if 0 < len(value):
        new_chain_symb = value

      new_coor_x = 0.0
      value = (indata[30:30+8]).strip()
      try:
        float(value)
        new_coor_x = value
      except:
        atom_data = False

      new_coor_y = 0.0
      value = (indata[38:38+8]).strip()
      try:
        float(value)
        new_coor_y = value
      except:
        atom_data = False

      new_coor_z = 0.0
      value = (indata[46:46+8]).strip()
      try:
        float(value)
        new_coor_z = value
      except:
        atom_data = False

      if not atom_data:
        continue

      if not cur_chain_symb == new_chain_symb:
        cur_chain_symb = new_chain_symb
        cur_resid_rank = 0
        cur_resid_atom = 0
      if chain_limit:
        if not cur_chain_symb in self.chainlist:
          continue

      if not cur_resid_rank == new_resid_rank:
        cur_resid_rank = new_resid_rank
        cur_resid_atom = 1
        cawas = False
      else:
        cur_resid_atom += 1
      if self.residstart > cur_resid_rank:
        continue
      if not 0 == self.residlast:
        if self.residlast < cur_resid_rank:
          continue

      to_print = False
      if 'A' == curlinetype:
        if "A" == self.atomtype:
          if "CA" == new_atom_symb:
            if not cawas:
              to_print = True
              cawas = True
        elif "B" == self.atomtype:
          if 5 > cur_resid_atom:
            to_print = True
        elif "C" == self.atomtype:
          if not new_atom_symb.startswith("H"):
            to_print = True
        else:
          to_print = True
      else:
        if "D" == self.atomtype:
          to_print = True

      if to_print:
        atom_rank = str(new_atom_rank)
        chain_symb = "-"
        if not "" == new_chain_symb:
          chain_symb = new_chain_symb
        resid_rank = str(new_resid_rank)
        resid_atom = str(cur_resid_atom)
        line_name = ":".join([curlinetype, atom_rank, chain_symb, resid_rank, new_resid_symb, resid_atom, new_atom_symb])
        line = ",".join([line_name, new_coor_x, new_coor_y, new_coor_z])
        print(line)

    fhpdb.close()
    return True


#params: -i input_file.pdb, -t atom_type, -c chains -s start_res -l last_res
if ('__main__' == __name__):
  args = sys.argv[1:]
  argl = len(args)
  runstring = "Should be run: pdb2csv.py -i file.pdb -t A|B|C|D -c chain_list "
  runstring += "-s start_residue -l last_residue\n"

  pdbfname = ""
  atomtype = "A"
  chainarg = ""
  chainlist = []
  residstart = 0
  residlast = 0

  optlist = []
  try:
    optlist, args = getopt.getopt(args, 'i:t:c:s:l:')
    for opt, arg in optlist:
      if "-i" == opt:
        pdbfname = arg
      if "-t" == opt:
        atomtype = arg
      if "-c" == opt:
        chainarg = arg
        chainlist = arg.split(":")
      if "-s" == opt:
        residstart = int(arg)
      if "-l" == opt:
        residlast = int(arg)

  except:
    sys.stderr.write(runstring)
    sys.exit(1)

  isok = True
  if 0 == len(pdbfname):
    isok = False
    sys.stderr.write("no input file\n")
  elif not os.path.isfile(pdbfname):
    isok = False
    sys.stderr.write("no such input file\n")
  elif not os.access(pdbfname, os.R_OK):
    isok = False
    sys.stderr.write("input file not readable\n")
  if not atomtype in ['A', 'B', 'C', 'D']:
    isok = False
    sys.stderr.write("unknown atom type\n")
  if 0 > residstart:
    isok = False
    sys.stderr.write("wrong start residuum rank\n")
  if 0 > residlast:
    isok = False
    sys.stderr.write("wrong last residuum rank\n")

  if not isok:
    sys.stderr.write(runstring)
    sys.exit(1)

  selc = SelCoords()
  selc.pdbfname = pdbfname
  selc.atomtype = atomtype
  selc.chainlist = chainlist
  selc.residsart = residstart
  selc.residlast = residlast

  params = ",".join([pdbfname, atomtype, chainarg, str(residstart), str(residlast)])
  print("#selcoords," + params)
  rv = selc.work()
  if not rv:
    print("work failed")
    sys.exit(1)

